@@ -25,9 +25,9 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <dirent.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include "menu-util.h"
+#include "menu-monitor.h"
 #include "canonicalize.h"
 
 typedef struct CachedDir CachedDir;
@@ -58,11 +58,13 @@ struct CachedDir
   GSList *entries;
   GSList *subdirs;
 
-  GnomeVFSMonitorHandle *monitor;
-  GSList                *monitors;
+  MenuMonitor *dir_monitor;
+  GSList      *monitors;
 
   guint have_read_entries : 1;
-  guint no_monitor_support : 1;
+  guint deleted : 1;
+
+  guint references : 28;
 };
 
 struct CachedDirMonitor
@@ -76,6 +78,11 @@ static void     cached_dir_free                   (CachedDir  *dir);
 static gboolean cached_dir_load_entries_recursive (CachedDir  *dir,
                                                    const char *dirname);
 
+static void handle_cached_dir_changed (MenuMonitor      *monitor,
+				       MenuMonitorEvent  event,
+				       const char       *path,
+				       CachedDir        *dir);
+
 /*
  * Entry directory cache
  */
@@ -95,38 +102,32 @@ cached_dir_new (const char *name)
 }
 
 static void
-cached_dir_clear_entries (CachedDir *dir)
+cached_dir_free (CachedDir *dir)
 {
+  if (dir->dir_monitor)
+    {
+      menu_monitor_remove_notify (dir->dir_monitor,
+				  (MenuMonitorNotifyFunc) handle_cached_dir_changed,
+				  dir);
+      menu_monitor_unref (dir->dir_monitor);
+      dir->dir_monitor = NULL;
+    }
+
+  g_slist_foreach (dir->monitors, (GFunc) g_free, NULL);
+  g_slist_free (dir->monitors);
+  dir->monitors = NULL;
+
   g_slist_foreach (dir->entries,
                    (GFunc) desktop_entry_unref,
                    NULL);
   g_slist_free (dir->entries);
   dir->entries = NULL;
-}
 
-static void
-cached_dir_clear_subdirs (CachedDir *dir)
-{
   g_slist_foreach (dir->subdirs,
                    (GFunc) cached_dir_free,
                    NULL);
   g_slist_free (dir->subdirs);
   dir->subdirs = NULL;
-}
-
-static void
-cached_dir_free (CachedDir *dir)
-{
-  cached_dir_clear_entries (dir);
-  cached_dir_clear_subdirs (dir);
-
-  if (dir->monitor)
-    gnome_vfs_monitor_cancel (dir->monitor);
-  dir->monitor = NULL;
-
-  g_slist_foreach (dir->monitors, (GFunc) g_free, NULL);
-  g_slist_free (dir->monitors);
-  dir->monitors = NULL;
 
   g_free (dir->name);
   g_free (dir);
@@ -183,9 +184,6 @@ cached_dir_find_relative_path (CachedDir  *dir,
   i = 0;
   while (split[i] != NULL)
     {
-      if (*(split[i]) == '\0')
-        continue;
-
       if (split[i + 1] != NULL)
         {
           if ((dir = find_subdir (dir, split[i])) == NULL)
@@ -205,74 +203,6 @@ cached_dir_find_relative_path (CachedDir  *dir,
   return retval;
 }
 
-static DesktopEntry *
-cached_dir_find_file_id (CachedDir  *dir,
-                         const char *file_id,
-                         gboolean    legacy)
-{
-  DesktopEntry *retval = NULL;
-
-  if (!legacy)
-    {
-      char *p;
-      char *freeme;
-
-      p = freeme = g_strdup (file_id);
-      while (p != NULL)
-        {
-          char *q;
-
-          if ((retval = find_entry (dir, p)))
-            break;
-
-          q = strchr (p, '-');
-          if (q)
-            {
-              CachedDir *subdir;
-
-              *(q++) = '\0';
-
-              if ((subdir = find_subdir (dir, p)))
-                {
-                  retval = cached_dir_find_file_id (subdir, q, legacy);
-                  if (retval)
-                    break;
-                }
-            }
-
-          p = q;
-        }
-
-      g_free (freeme);
-    }
-  else /* legacy */
-    {
-      retval = find_entry (dir, file_id);
-
-      /* Ignore entries with categories in legacy trees
-       */
-      if (retval && desktop_entry_has_categories (retval))
-        retval = NULL;
-
-      if (!retval)
-        {
-          GSList *tmp;
-
-          tmp = dir->subdirs;
-          while (tmp != NULL)
-            {
-              retval = cached_dir_find_file_id (tmp->data, file_id, legacy);
-              if (retval)
-                break;
-
-              tmp = tmp->next;
-            }
-        }
-    }
-
-  return retval;
-}
-
 static CachedDir *
 cached_dir_lookup (const char *canonical)
 {
@@ -284,7 +214,7 @@ cached_dir_lookup (const char *canonical)
     dir_cache = cached_dir_new ("/");
   dir = dir_cache;
 
-  g_assert (canonical != NULL && canonical[0] == '/');
+  g_assert (canonical != NULL && canonical[0] == G_DIR_SEPARATOR);
 
   menu_verbose ("Looking up cached dir \"%s\"\n", canonical);
 
@@ -295,9 +225,6 @@ cached_dir_lookup (const char *canonical)
     {
       CachedDir *subdir;
 
-      if (*(split[i]) == '\0')
-        continue;
-
       if ((subdir = find_subdir (dir, split[i])) == NULL)
         {
           subdir = cached_dir_new (split[i]);
@@ -387,6 +314,21 @@ cached_dir_add_subdir (CachedDir  *dir,
                        const char *path)
 {
   CachedDir *subdir;
+  GSList    *tmp;
+
+  tmp = dir->subdirs;
+  while (tmp != NULL)
+    {
+      subdir = (CachedDir *) tmp->data;
+
+      if (strcmp (subdir->name, basename) == 0)
+        {
+	  subdir->deleted = FALSE;
+          return TRUE;
+        }
+
+      tmp = tmp->next;
+    }
 
   subdir = cached_dir_new (basename);
 
@@ -417,8 +359,14 @@ cached_dir_remove_subdir (CachedDir  *dir,
 
       if (strcmp (subdir->name, basename) == 0)
         {
-	  cached_dir_free (tmp->data);
-          dir->subdirs = g_slist_delete_link (dir->subdirs, tmp);
+	  subdir->deleted = TRUE;
+
+	  if (subdir->references == 0)
+	    {
+	      cached_dir_free (subdir);
+	      dir->subdirs = g_slist_delete_link (dir->subdirs, tmp);
+	    }
+
           return TRUE;
         }
 
@@ -451,46 +399,37 @@ cached_dir_invoke_monitors (CachedDir *dir)
 }
 
 static void
-handle_cached_dir_changed (GnomeVFSMonitorHandle    *handle,
-                           const char               *monitor_uri,
-                           const char               *info_uri,
-                           GnomeVFSMonitorEventType  event,
-                           CachedDir                *dir)
+handle_cached_dir_changed (MenuMonitor      *monitor,
+			   MenuMonitorEvent  event,
+			   const char       *path,
+                           CachedDir        *dir)
 {
   gboolean  handled = FALSE;
-  char     *path;
   char     *basename;
+  char     *dirname;
 
-  if (event != GNOME_VFS_MONITOR_EVENT_CREATED &&
-      event != GNOME_VFS_MONITOR_EVENT_DELETED &&
-      event != GNOME_VFS_MONITOR_EVENT_CHANGED)
-    return;
-
-  if (!(path = gnome_vfs_get_local_path_from_uri (info_uri)))
-    return;
-
-  menu_verbose ("Notified of '%s' %s - invalidating cache\n",
+  menu_verbose ("'%s' notified of '%s' %s - invalidating cache\n",
+		dir->name,
                 path,
-                event == GNOME_VFS_MONITOR_EVENT_CREATED ? ("created") :
-                event == GNOME_VFS_MONITOR_EVENT_DELETED ? ("deleted") :
-                event == GNOME_VFS_MONITOR_EVENT_CHANGED ? ("changed") : ("unknown-event"));
+                event == MENU_MONITOR_EVENT_CREATED ? ("created") :
+                event == MENU_MONITOR_EVENT_DELETED ? ("deleted") : ("changed"));
 
+  dirname  = g_path_get_dirname  (path);
   basename = g_path_get_basename (path);
 
+  dir = cached_dir_lookup (dirname);
+
   if (g_str_has_suffix (basename, ".desktop") ||
       g_str_has_suffix (basename, ".directory"))
     {
       switch (event)
         {
-        case GNOME_VFS_MONITOR_EVENT_CREATED:
-          handled = cached_dir_add_entry (dir, basename, path);
-          break;
-
-        case GNOME_VFS_MONITOR_EVENT_CHANGED:
+        case MENU_MONITOR_EVENT_CREATED:
+        case MENU_MONITOR_EVENT_CHANGED:
           handled = cached_dir_update_entry (dir, basename, path);
           break;
 
-        case GNOME_VFS_MONITOR_EVENT_DELETED:
+        case MENU_MONITOR_EVENT_DELETED:
           handled = cached_dir_remove_entry (dir, basename);
           break;
 
@@ -503,14 +442,14 @@ handle_cached_dir_changed (GnomeVFSMonitorHandle    *handle,
     {
       switch (event)
         {
-        case GNOME_VFS_MONITOR_EVENT_CREATED:
+        case MENU_MONITOR_EVENT_CREATED:
           handled = cached_dir_add_subdir (dir, basename, path);
           break;
 
-        case GNOME_VFS_MONITOR_EVENT_CHANGED:
+        case MENU_MONITOR_EVENT_CHANGED:
           break;
 
-        case GNOME_VFS_MONITOR_EVENT_DELETED:
+        case MENU_MONITOR_EVENT_DELETED:
           handled = cached_dir_remove_subdir (dir, basename);
           break;
 
@@ -521,7 +460,7 @@ handle_cached_dir_changed (GnomeVFSMonitorHandle    *handle,
     }
 
   g_free (basename);
-  g_free (path);
+  g_free (dirname);
 
   if (handled)
     {
@@ -533,23 +472,13 @@ static void
 cached_dir_ensure_monitor (CachedDir  *dir,
                            const char *dirname)
 {
-  char *uri;
-
-  if (dir->monitor != NULL || dir->no_monitor_support)
-    return;
-
-  uri = gnome_vfs_get_uri_from_local_path (dirname);
-
-  if (gnome_vfs_monitor_add (&dir->monitor,
-                             uri,
-                             GNOME_VFS_MONITOR_DIRECTORY,
-                             (GnomeVFSMonitorCallback) handle_cached_dir_changed,
-                             dir) != GNOME_VFS_OK)
+  if (dir->dir_monitor == NULL)
     {
-      dir->no_monitor_support = TRUE;
+      dir->dir_monitor = menu_get_directory_monitor (dirname);
+      menu_monitor_add_notify (dir->dir_monitor,
+			       (MenuMonitorNotifyFunc) handle_cached_dir_changed,
+			       dir);
     }
-
-  g_free (uri);
 }
 
 static gboolean
@@ -577,12 +506,11 @@ cached_dir_load_entries_recursive (CachedDir  *dir,
       return FALSE;
     }
 
-  cached_dir_clear_entries (dir);
   cached_dir_ensure_monitor (dir, dirname);
 
   fullpath = g_string_new (dirname);
-  if (fullpath->str[fullpath->len - 1] != '/')
-    g_string_append_c (fullpath, '/');
+  if (fullpath->str[fullpath->len - 1] != G_DIR_SEPARATOR)
+    g_string_append_c (fullpath, G_DIR_SEPARATOR);
 
   fullpath_len = fullpath->len;
 
@@ -619,52 +547,6 @@ cached_dir_load_entries_recursive (CachedDir  *dir,
   return TRUE;
 }
 
-static char *
-cached_dir_get_full_path (CachedDir *dir)
-{
-  GString *str;
-  GSList  *parents = NULL;
-  GSList  *tmp;
-
-  if (dir == NULL || dir->parent == NULL)
-    return g_strdup ("/");
-
-  while (dir->parent != NULL)
-    {
-      parents = g_slist_prepend (parents, dir->name);
-      dir = dir->parent;
-    }
-
-  str = g_string_new ("/");
-
-  tmp = parents;
-  while (tmp != NULL)
-    {
-      g_string_append (str, tmp->data);
-      g_string_append_c (str, '/');
-
-      tmp = tmp->next;
-    }
-
-  g_slist_free (parents);
-
-  return g_string_free (str, FALSE);
-}
-
-static CachedDir *
-cached_dir_load (const char *canonical_path)
-{
-  CachedDir *retval;
-
-  menu_verbose ("Loading cached dir \"%s\"\n", canonical_path);
-
-  retval = cached_dir_lookup (canonical_path);
-
-  cached_dir_load_entries_recursive (retval, canonical_path);
-
-  return retval;
-}
-
 static void
 cached_dir_add_monitor (CachedDir                 *dir,
                         EntryDirectory            *ed,
@@ -725,32 +607,51 @@ cached_dir_remove_monitor (CachedDir                 *dir,
 }
 
 static void
-cached_dir_ensure_loaded (CachedDir *dir)
+cached_dir_add_reference (CachedDir *dir)
 {
-  char *path;
+  dir->references++;
 
-  if (dir->have_read_entries)
-    return;
-
-  path = cached_dir_get_full_path (dir);
-  cached_dir_load_entries_recursive (dir, path);
-  g_free (path);
+  if (dir->parent != NULL)
+    {
+      cached_dir_add_reference (dir->parent);
+    }
 }
 
-static GSList*
-cached_dir_get_subdirs (CachedDir *dir)
+static void
+cached_dir_remove_reference (CachedDir *dir)
 {
-  cached_dir_ensure_loaded (dir);
+  CachedDir *parent;
 
-  return dir->subdirs;
-}
+  parent = dir->parent;
 
-static GSList*
-cached_dir_get_entries (CachedDir   *dir)
-{
-  cached_dir_ensure_loaded (dir);
+  if (--dir->references == 0 && dir->deleted)
+    {
+      if (dir->parent != NULL)
+	{
+	  GSList *tmp;
 
-  return dir->entries;
+	  tmp = parent->subdirs;
+	  while (tmp != NULL)
+	    {
+	      CachedDir *subdir = tmp->data;
+
+	      if (!strcmp (subdir->name, dir->name))
+		{
+		  parent->subdirs = g_slist_delete_link (parent->subdirs, tmp);
+		  break;
+		}
+
+	      tmp = tmp->next;
+	    }
+	}
+
+      cached_dir_free (dir);
+    }
+
+  if (parent != NULL)
+    {
+      cached_dir_remove_reference (parent);
+    }
 }
 
 /*
@@ -764,7 +665,6 @@ entry_directory_new_full (DesktopEntryType  entry_type,
                           const char       *legacy_prefix)
 {
   EntryDirectory *ed;
-  CachedDir      *cd;
   char           *canonical;
 
   menu_verbose ("Loading entry directory \"%s\" (legacy %s)\n",
@@ -779,12 +679,14 @@ entry_directory_new_full (DesktopEntryType  entry_type,
       return NULL;
     }
 
-  cd = cached_dir_load (canonical);
-  g_assert (cd != NULL);
-
   ed = g_new0 (EntryDirectory, 1);
 
-  ed->dir           = cd;
+  ed->dir = cached_dir_lookup (canonical);
+  g_assert (ed->dir != NULL);
+
+  cached_dir_add_reference (ed->dir);
+  cached_dir_load_entries_recursive (ed->dir, canonical);
+
   ed->legacy_prefix = g_strdup (legacy_prefix);
   ed->entry_type    = entry_type;
   ed->is_legacy     = is_legacy != FALSE;
@@ -829,6 +731,8 @@ entry_directory_unref (EntryDirectory *ed)
 
   if (--ed->refcount == 0)
     {
+      cached_dir_remove_reference (ed->dir);
+
       ed->dir        = NULL;
       ed->entry_type = DESKTOP_ENTRY_INVALID;
       ed->is_legacy  = FALSE;
@@ -856,47 +760,6 @@ entry_directory_remove_monitor (EntryDirectory            *ed,
   cached_dir_remove_monitor (ed->dir, ed, callback, user_data);
 }
 
-static DesktopEntry *
-entry_directory_get_desktop (EntryDirectory *ed,
-                             const char     *file_id)
-{
-  DesktopEntry *entry;
-  const char   *unprefixed_id;
-
-  if (ed->entry_type != DESKTOP_ENTRY_DESKTOP)
-    return NULL;
-
-  unprefixed_id = NULL;
-
-  if (ed->is_legacy && ed->legacy_prefix)
-    {
-      if (!g_str_has_prefix (file_id, ed->legacy_prefix))
-        return NULL;
-
-      unprefixed_id = file_id + strlen (ed->legacy_prefix);
-
-      if (unprefixed_id[0] != '-')
-	return NULL;
-
-      unprefixed_id++;
-    }
-
-  entry = cached_dir_find_file_id (ed->dir,
-                                   unprefixed_id ? unprefixed_id : file_id,
-                                   ed->is_legacy);
-  if (entry == NULL || desktop_entry_get_type (entry) != DESKTOP_ENTRY_DESKTOP)
-    return NULL;
-
-  if (ed->is_legacy && !desktop_entry_has_categories (entry))
-    {
-      entry = desktop_entry_copy (entry);
-      desktop_entry_add_legacy_category (entry);
-      return entry;
-    }
-
-  return desktop_entry_ref (entry);
-}
-
 static DesktopEntry *
 entry_directory_get_directory (EntryDirectory *ed,
                                const char     *relative_path)
@@ -914,30 +777,40 @@ entry_directory_get_directory (EntryDirectory *ed,
 }
 
 static char *
-get_desktop_file_id_from_path (EntryDirectory *ed,
-                               const char     *relative_path)
+get_desktop_file_id_from_path (EntryDirectory   *ed,
+			       DesktopEntryType  entry_type,
+			       const char       *relative_path)
 {
   char *retval;
+  
+  retval = NULL;
 
-  if (!ed->is_legacy)
+  if (entry_type == DESKTOP_ENTRY_DESKTOP)
     {
-      retval = g_strdelimit (g_strdup (relative_path), "/", '-');
+      if (!ed->is_legacy)
+	{
+	  retval = g_strdelimit (g_strdup (relative_path), "/", '-');
+	}
+      else
+	{
+	  char *basename;
+
+	  basename = g_path_get_basename (relative_path);
+
+	  if (ed->legacy_prefix)
+	    {
+	      retval = g_strjoin ("-", ed->legacy_prefix, basename, NULL);
+	      g_free (basename);
+	    }
+	  else
+	    {
+	      retval = basename;
+	    }
+	}
     }
   else
     {
-      char *basename;
-
-      basename = g_path_get_basename (relative_path);
-
-      if (ed->legacy_prefix)
-        {
-          retval = g_strjoin ("-", ed->legacy_prefix, basename, NULL);
-          g_free (basename);
-        }
-      else
-        {
-          retval = basename;
-        }
+      retval = g_strdup (relative_path);
     }
 
   return retval;
@@ -945,7 +818,6 @@ get_desktop_file_id_from_path (EntryDirectory *ed,
 
 typedef gboolean (* EntryDirectoryForeachFunc) (EntryDirectory  *ed,
                                                 DesktopEntry    *entry,
-                                                const char      *relative_path,
                                                 const char      *file_id,
                                                 DesktopEntrySet *set,
                                                 gpointer         user_data);
@@ -961,9 +833,12 @@ entry_directory_foreach_recursive (EntryDirectory            *ed,
   GSList *tmp;
   int     relative_path_len;
 
+  if (cd->deleted)
+    return TRUE;
+
   relative_path_len = relative_path->len;
 
-  tmp = cached_dir_get_entries (cd);
+  tmp = cd->entries;
   while (tmp != NULL)
     {
       DesktopEntry *entry = tmp->data;
@@ -976,11 +851,11 @@ entry_directory_foreach_recursive (EntryDirectory            *ed,
           g_string_append (relative_path,
                            desktop_entry_get_basename (entry));
 
-          file_id = NULL;
-          if (ed->entry_type == DESKTOP_ENTRY_DESKTOP)
-            file_id = get_desktop_file_id_from_path (ed, relative_path->str);
+	  file_id = get_desktop_file_id_from_path (ed,
+						   ed->entry_type,
+						   relative_path->str);
 
-          ret = func (ed, entry, relative_path->str, file_id, set, user_data);
+          ret = func (ed, entry, file_id, set, user_data);
 
           g_free (file_id);
 
@@ -993,13 +868,13 @@ entry_directory_foreach_recursive (EntryDirectory            *ed,
       tmp = tmp->next;
     }
 
-  tmp = cached_dir_get_subdirs (cd);
+  tmp = cd->subdirs;
   while (tmp != NULL)
     {
       CachedDir *subdir = tmp->data;
 
       g_string_append (relative_path, subdir->name);
-      g_string_append_c (relative_path, '/');
+      g_string_append_c (relative_path, G_DIR_SEPARATOR);
 
       if (!entry_directory_foreach_recursive (ed,
                                               subdir,
@@ -1048,7 +923,7 @@ entry_directory_get_flat_contents (EntryDirectory   *ed,
   if (subdirs)
     *subdirs = NULL;
 
-  tmp = cached_dir_get_entries (ed->dir);
+  tmp = ed->dir->entries;
   while (tmp != NULL)
     {
       DesktopEntry *entry = tmp->data;
@@ -1061,7 +936,9 @@ entry_directory_get_flat_contents (EntryDirectory   *ed,
         {
           char *file_id;
 
-          file_id = get_desktop_file_id_from_path (ed, basename);
+          file_id = get_desktop_file_id_from_path (ed,
+						   DESKTOP_ENTRY_DESKTOP,
+						   basename);
 
           desktop_entry_set_add_entry (desktop_entries,
                                        entry,
@@ -1074,8 +951,8 @@ entry_directory_get_flat_contents (EntryDirectory   *ed,
           desktop_entry_get_type (entry) == DESKTOP_ENTRY_DIRECTORY)
         {
           desktop_entry_set_add_entry (directory_entries,
-                                       entry,
-                                       g_strdup (basename));
+				       entry,
+				       basename);
         }
 
       tmp = tmp->next;
@@ -1083,12 +960,15 @@ entry_directory_get_flat_contents (EntryDirectory   *ed,
 
   if (subdirs)
     {
-      tmp = cached_dir_get_subdirs (ed->dir);
+      tmp = ed->dir->subdirs;
       while (tmp != NULL)
         {
           CachedDir *cd = tmp->data;
 
-          *subdirs = g_slist_prepend (*subdirs, g_strdup (cd->name));
+	  if (!cd->deleted)
+	    {
+	      *subdirs = g_slist_prepend (*subdirs, g_strdup (cd->name));
+	    }
 
           tmp = tmp->next;
         }
@@ -1183,25 +1063,6 @@ entry_directory_list_append_list (EntryDirectoryList *list,
   list->dirs = g_list_concat (list->dirs, new_dirs);
 }
 
-DesktopEntry *
-entry_directory_list_get_desktop (EntryDirectoryList *list,
-                                  const char         *file_id)
-{
-  DesktopEntry *retval = NULL;
-  GList        *tmp;
-
-  tmp = list->dirs;
-  while (tmp != NULL)
-    {
-      if ((retval = entry_directory_get_desktop (tmp->data, file_id)) != NULL)
-        break;
-
-      tmp = tmp->next;
-    }
-
-  return retval;
-}
-
 DesktopEntry *
 entry_directory_list_get_directory (EntryDirectoryList *list,
                                     const char         *relative_path)
@@ -1221,14 +1082,38 @@ entry_directory_list_get_directory (EntryDirectoryList *list,
   return retval;
 }
 
-static void
-entry_directory_list_add (EntryDirectoryList        *list,
-                          EntryDirectoryForeachFunc  func,
-                          DesktopEntrySet           *set,
-                          gpointer                   user_data)
+static gboolean
+get_all_func (EntryDirectory   *ed,
+              DesktopEntry     *entry,
+              const char       *file_id,
+              DesktopEntrySet  *set,
+              gpointer          user_data)
+{
+  if (ed->is_legacy && !desktop_entry_has_categories (entry))
+    {
+      entry = desktop_entry_copy (entry);
+      desktop_entry_add_legacy_category (entry);
+    }
+  else
+    {
+      entry = desktop_entry_ref (entry);
+    }
+
+  desktop_entry_set_add_entry (set, entry, file_id);
+  desktop_entry_unref (entry);
+
+  return TRUE;
+}
+
+void
+entry_directory_list_get_all_desktops (EntryDirectoryList *list,
+                                       DesktopEntrySet    *set)
 {
   GList *tmp;
 
+  menu_verbose (" Storing all of list %p in set %p\n",
+                list, set);
+
   /* The only tricky thing here is that desktop files later
    * in the search list with the same relative path
    * are "hidden" by desktop files earlier in the path,
@@ -1245,48 +1130,12 @@ entry_directory_list_add (EntryDirectoryList        *list,
   tmp = g_list_last (list->dirs);
   while (tmp != NULL)
     {
-      entry_directory_foreach (tmp->data, func, set, user_data);
+      entry_directory_foreach (tmp->data, get_all_func, set, NULL);
 
       tmp = tmp->prev;
     }
 }
 
-static gboolean
-get_all_func (EntryDirectory   *ed,
-              DesktopEntry     *entry,
-              const char       *relative_path,
-              const char       *file_id,
-              DesktopEntrySet  *set,
-              gpointer          user_data)
-{
-  if (ed->is_legacy && !desktop_entry_has_categories (entry))
-    {
-      entry = desktop_entry_copy (entry);
-      desktop_entry_add_legacy_category (entry);
-    }
-  else
-    {
-      entry = desktop_entry_ref (entry);
-    }
-
-  desktop_entry_set_add_entry (set,
-                               entry,
-                               file_id ? file_id : relative_path);
-  desktop_entry_unref (entry);
-
-  return TRUE;
-}
-
-void
-entry_directory_list_get_all_desktops (EntryDirectoryList *list,
-                                       DesktopEntrySet    *set)
-{
-  menu_verbose (" Storing all of list %p in set %p\n",
-                list, set);
-
-  entry_directory_list_add (list, get_all_func, set, NULL);
-}
-
 void
 entry_directory_list_add_monitors (EntryDirectoryList        *list,
                                    EntryDirectoryChangedFunc  callback,
